// Run graphcuts using spatially-varying smoothness costs (diagonal matrices)

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "GCoptimization.h"
#include "mex.h"

Graph::captype smoothFn(int p1, int p2, int l1, int l2, void *SmoothnessCost) {
  if(l1!=l2) return 0;

  Graph::captype *s = (Graph::captype*)SmoothnessCost;
  int N = (int)s[0];
  int width = (int)s[1];
  int height = (int)s[2];
  s = s+3;

  int x,y;
  if(p1+1==p2) { // horizontal neighbors
    x = p1%width;
    y = p1/width;
    return s[l1+x*N+y*N*(width-1)];
//     return s[l1+l2*N+x*N*N+y*N*N*(width-1)];
  }
  if(p2+1==p1) { // horizontal neighbors
    x = p2%width;
    y = p2/width;
    return s[l1+x*N+y*N*(width-1)];
//     return s[l1+l2*N+x*N*N+y*N*N*(width-1)];
  }
  if(p1+width==p2) { // vertical neighbors
    x = p1%width;
    y = p1/width;
    return s[N*(width-1)*height+l1+x*N+y*N*width];
//     return s[N*N*(width-1)*height+l1+l2*N+x*N*N+y*N*N*width];
  }
  if(p2+width==p1) { // vertical neighbors
    x = p2%width;
    y = p2/width;
    return s[N*(width-1)*height+l1+x*N+y*N*width];
//     return s[N*N*(width-1)*height+l1+l2*N+x*N*N+y*N*N*width];
  }
  else {
    fprintf(stdout,"Other connection detected!\n");
    fflush(stdout);
    return (l1==l2) ? -0.1 : 0;
  }
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {
  if(nrhs != 2) {
    mexErrMsgTxt("Error: 2 args. needed.");
    return;
  }

  Graph::captype *DataCost = (Graph::captype*)mxGetData(prhs[0]);
  Graph::captype *SmoothnessCost = (Graph::captype*)mxGetData(prhs[1]);

  const mwSize *dims = mxGetDimensions(prhs[0]);
  int nlabels = dims[0];
  int ncols = dims[1];
  int nrows = dims[2];
  int num_pixels = nrows*ncols;

  // Allocate outputs:
  plhs[0] = mxCreateNumericMatrix(ncols,nrows,mxINT32_CLASS,mxREAL);
  int *result = (int*)mxGetData(plhs[0]);

  GCoptimizationGridGraph *gc = new GCoptimizationGridGraph(ncols,nrows,nlabels);
    
  gc->setDataCost(DataCost);

  // Set spatially-varying smoothness costs:
  gc->setSmoothCost(&smoothFn,SmoothnessCost);
//   gc->setSmoothCost(SmoothnessCost);

  printf("\nBefore optimization energy is %f\n",gc->compute_energy());
  gc->swap();
  printf("\nAfter optimization energy is %f\n",gc->compute_energy());
  
  for ( int  i = 0; i < num_pixels; i++ )
    result[i] = gc->whatLabel(i);
  
  delete gc;
}
