// Run graphcuts using spatially-varying smoothness costs.

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "GCoptimization.h"
#include "mex.h"

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {
  if(nrhs != 2) {
    mexErrMsgTxt("Error: 2 args. needed.");
    return;
  }

  Graph::captype *DataCost = (Graph::captype*)mxGetData(prhs[0]);
  Graph::captype *SmoothnessCost = (Graph::captype*)mxGetData(prhs[1]);

  const mwSize *dims = mxGetDimensions(prhs[0]);
  int nlabels = dims[0];
  int ncols = dims[1];
  int nrows = dims[2];
  int num_pixels = nrows*ncols;

  // Allocate outputs:
  plhs[0] = mxCreateNumericMatrix(ncols,nrows,mxINT32_CLASS,mxREAL);
  int *result = (int*)mxGetData(plhs[0]);

  GCoptimizationGridGraph *gc = new GCoptimizationGridGraph(ncols,nrows,nlabels);
    
  gc->setDataCost(DataCost);

  // Set spatially-varying smoothness costs:
  gc->setSmoothCost(SmoothnessCost);

  printf("\nBefore optimization energy is %d",gc->compute_energy());
  gc->swap();
  printf("\nAfter optimization energy is %d",gc->compute_energy());
  
  for ( int  i = 0; i < num_pixels; i++ )
    result[i] = gc->whatLabel(i);
  
  delete gc;
}
